/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/
/*
 *  Created by Ruben Zilibowitz on 16/08/06.
 */

#include <stdio.h>
#include "TrackEditor.h"
#include "ScTilemap.h"
#include "ScView.h"
#include "ScException.h"
#include "TcPreferences.h"
#include "PathLine.h"
#include "Startline.h"
#include "GameConstants.h"
#include "tinyxml.h"
#include "main.h"
#include "Minimap.h"
#include "glf.h"
#include <SDL.h>
#include <SDL_image.h>

extern TcPreferences *gPreferences;

ScTilemap *gTiles;
Minimap *gMinimap;
char gLabel[256];
double gLabelTime = -1.0;

const unsigned int kMinimapUpdateLimit = 10;
const double kDefaultLabelTime = 8.0;

void showHelp();
void MainLoop(ScView& view, list<Tuple>& pathPolygon, Startline *startLine);
void saveTrack(const char *trackname, list<Tuple>& pathPolygon, Tuple startLineA, Tuple startLineB);
void loadTrack(const char *trackname, list<Tuple>& pathPolygon, Startline *startLine);
void loadFromImage(const char *imagefilename);
void installTrack(const char *name);
void uninstallTrack(const char *name);

void runTrackEditor()
{
	printf("Running Track Editor...\n");
	printf("Left click - draw grass\n");
	printf("Right click - draw wall\n");
	printf("l - load track (give path)\n");
	printf("s - save track (give path)\n");
	printf("m - load map from 256 colours image file (give path).\n");
	printf("    Black is grass you can drive on. Any other colour is a wall.\n");
	printf("c - begin/end of startline\n");
	printf("p - add a new point to the path polygon\n");
	printf("shift p - remove the last point from the path polygon\n");
	printf("i - install track into data directory (give only name)\n");
	printf("u - uninstall a track (give only name)\n");
	printf("h - display help screen\n");
	printf("escape - exit track editor\n");
	
	ScTilemap* tilemap = loadTileset("default");	// will get deleted when animation destructor is called
	ScAnimation animation;
	animation.push_back(tilemap);
	gTiles = tilemap;
	ScPortRect portRect = {0, 0, gPreferences->getScreenWidth(), gPreferences->getScreenHeight()};
	ScRect viewRect = {0, 0, portRect.width, portRect.height};
	ScView view(portRect, viewRect, animation);
	tilemap->newMap(80, 50, 25);
	
	list<Player*> emptyPlayersList;
	gMinimap = new Minimap(*tilemap, emptyPlayersList);
	
	list<Tuple> pathPolygon;
	animation.push_back(new PathLine(tilemap->getBounds(), &pathPolygon));
	
	Startline *startLine = new Startline(Tuple(viewRect.right*0.5,viewRect.top*0.75+viewRect.bottom*0.25),
										 Tuple(viewRect.right*0.5,viewRect.top*0.25+viewRect.bottom*0.75),
										 kDefaultStartLineWidth, true);
	animation.push_back(startLine);
	
	gLabelTime = -1.0;
	MainLoop(view, pathPolygon, startLine);
}

void showLabel(const char *label, double time = kDefaultLabelTime)
{
	strncpy(gLabel, label, 255);
	gLabelTime = time;
}

void updateLabel(double dt)
{
	if (gLabelTime > 0)
	{
		glDisable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		glColor4f(1.0f,1.0f,1.0f,0.5f);
		glLoadIdentity();
		glOrtho(0, gPreferences->getScreenWidth(), 0, gPreferences->getScreenHeight(), -1, 1);
		glViewport(0, 0, gPreferences->getScreenWidth(), gPreferences->getScreenHeight());
		glScalef(10, 10, 1);
		glTranslatef(4, 4, 0);
		glfDrawSolidString(gLabel);
		gLabelTime -= dt;
	}
}

void showHelp()
{
	SDL_Event event;
	
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glColor4f(1.0f,1.0f,1.0f,0.5f);
	glLoadIdentity();
	glOrtho(0, gPreferences->getScreenWidth(), 0, gPreferences->getScreenHeight(), -1, 1);
	glViewport(0, 0, gPreferences->getScreenWidth(), gPreferences->getScreenHeight());
	glScalef(10, 10, 1);
	glTranslatef(4, 48, 0);
	
	glfDrawSolidString("Track Editor Help...");
	glTranslatef(0, -4, 0);
	glfDrawSolidString("Left click - draw grass");
	glTranslatef(0, -2, 0);
	glfDrawSolidString("Right click - draw wall");
	glTranslatef(0, -2, 0);
	glfDrawSolidString("l - load track (give path)");
	glTranslatef(0, -2, 0);
	glfDrawSolidString("s - save track (give path)");
	glTranslatef(0, -2, 0);
	glfDrawSolidString("m - load map from 256 colours image file (give path).");
	glTranslatef(0, -2, 0);
	glfDrawSolidString("  Black is grass you can drive on. Any other colour is a wall.");
	glTranslatef(0, -2, 0);
	glfDrawSolidString("c - begin/end of startline");
	glTranslatef(0, -2, 0);
	glfDrawSolidString("p - add a new point to the path polygon");
	glTranslatef(0, -2, 0);
	glfDrawSolidString("shift p - remove the last point from the path polygon");
	glTranslatef(0, -2, 0);
	glfDrawSolidString("i - install track into data directory (give only name)");
	glTranslatef(0, -2, 0);
	glfDrawSolidString("u - uninstall a track (give only name)");
	glTranslatef(0, -2, 0);
	glfDrawSolidString("h - display help screen");
	glTranslatef(0, -2, 0);
	glfDrawSolidString("escape - exit track editor");
	glTranslatef(0, -4, 0);
	glfDrawSolidString("Press any key to continue.");
	SDL_GL_SwapBuffers();
	
	while (SDL_WaitEvent(&event))
	{
		if (event.type == SDL_KEYDOWN)
			break;
	}
}

char *getName(ScView& view)
{
/*
	char str[256];
	printf("Please enter the name (including path): ");
	fgets(str, 256, stdin);
	str[strlen(str)-1] = '\0';	// strips the tailing newline character
	return strdup(str);
*/
	SDL_Event event;
	int i = 0;
	char printStr[300] = "";
	char str255[256] = "";
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glColor4f(1.0f,1.0f,1.0f,0.5f);
	glLoadIdentity();
	glOrtho(0, gPreferences->getScreenWidth(), 0, gPreferences->getScreenHeight(), -1, 1);
	glViewport(0, 0, gPreferences->getScreenWidth(), gPreferences->getScreenHeight());
	glTranslatef(24, 48, 0);
	glScalef(10, 10, 1);
	
	while (SDL_WaitEvent(&event))
	{
		glPushMatrix();
		view.draw();
		glPopMatrix();
		sprintf(printStr, "Enter name: %s", str255);
		glfDrawSolidString(printStr);
		SDL_GL_SwapBuffers();
		
		if (event.type == SDL_KEYDOWN)
		{
			if (event.key.keysym.sym == SDLK_ESCAPE)
				return NULL;
			if (event.key.keysym.sym == SDLK_RETURN || event.key.keysym.sym == SDLK_KP_ENTER)
			{
				break;
			}
			else if (event.key.keysym.sym == SDLK_BACKSPACE && i > 0)
			{
				i--;
			}
			if (event.key.keysym.sym >= SDLK_SPACE && event.key.keysym.sym <= SDLK_z && i < 255)
			{
				char c = event.key.keysym.sym;
				if (event.key.keysym.mod & KMOD_SHIFT)
					c = toupper(c);
				str255[i] = c;
				i++;
			}
			str255[i] = '\0';
		}
	}
	
	return strdup(str255);
}

void MainLoop(ScView& view, list<Tuple>& pathPolygon, Startline *startLine)
{
	bool running;
	SDL_Event event;
	const short screenWidth = gPreferences->getScreenWidth();
	const short screenHeight = gPreferences->getScreenHeight();
	const short scrollBorder = 60;
	bool scrollLeft = false, scrollRight = false, scrollUp = false, scrollDown = false;
	bool buttonState = false, leftB = false, rightB = false;
	int mousex;
	int mousey;
	ScFloat xloc;
	ScFloat yloc;
	bool drawingStartLine = false;
	char *buffer;
	unsigned int tileToggleCount = 0;
	Tuple startLineA(0,0);
	
	// Loop, drawing and checking events
	running = true;
	while (running)
	{
		view.draw();
		gMinimap->draw(10, 10, 2.0);
		updateLabel(0.01);
		SDL_GL_SwapBuffers();
		
		SDL_GetMouseState(&mousex, &mousey);
		xloc = mousex + view.getViewRect().left - gTiles->getBounds().left;
		yloc = view.getViewRect().top - mousey - gTiles->getBounds().bottom;
		
		// get pending events
		while (SDL_PollEvent(&event))
		{
			if (event.type == SDL_QUIT)
				running = false;
			if (event.type == SDL_MOUSEMOTION)
			{
				scrollLeft = event.motion.x < scrollBorder;
				scrollRight = event.motion.x > screenWidth - scrollBorder;
				scrollUp = event.motion.y < scrollBorder;
				scrollDown = event.motion.y > screenHeight - scrollBorder;
			}
			if (event.type == SDL_MOUSEBUTTONDOWN)
			{
				buttonState = true;
				rightB = (event.button.button == SDL_BUTTON_RIGHT);
				leftB = (event.button.button == SDL_BUTTON_LEFT);
			}
			if (event.type == SDL_MOUSEBUTTONUP)
			{
				buttonState = false;
			}
			if (event.type == SDL_KEYDOWN)
			{
				switch (event.key.keysym.sym)
				{
					case SDLK_p:
						if (event.key.keysym.mod & KMOD_SHIFT) {
							if (!pathPolygon.empty())
								pathPolygon.pop_back();
						}
						else
							pathPolygon.push_back(Tuple(xloc/kScale,yloc/kScale));
						break;
					case SDLK_c:
						drawingStartLine = !drawingStartLine;
						startLineA.x = xloc;
						startLineA.y = yloc;
						break;
					case SDLK_l:
						printf("Loading tilemap...\n");
						buffer = getName(view);
						if (buffer == NULL)	break;
						try { loadTrack(buffer, pathPolygon, startLine); }
						catch (ScException e) {
							e.printMsg();
							showLabel("Error loading track");
						}
						free(buffer);
						gMinimap->updateMap(*gTiles);
						break;
					case SDLK_m:
						printf("Loading tilemap from image...\n");
						buffer = getName(view);
						if (buffer == NULL)	break;
						try { loadFromImage(buffer); }
						catch (ScException e) {
							e.printMsg();
							showLabel("Error loading tilemap from image");
						}
						free(buffer);
						gMinimap->updateMap(*gTiles);
						break;
					case SDLK_s:
						printf("Saving tilemap...\n");
						buffer = getName(view);
						if (buffer == NULL)	break;
						try { saveTrack(buffer, pathPolygon, startLine->getStartPoint(), startLine->getEndPoint()); }
						catch (ScException e) {
							e.printMsg();
							showLabel("Error saving tilemap");
						}
						free(buffer);
						break;
					case SDLK_i:
						buffer = getName(view);
						if (buffer == NULL)	break;
						{
							char path[256];
							//strcpy(path, DATA_DIR);
							strcpy(path, "tracks/");
							strcat(path, buffer);
							try {
								saveTrack(path, pathPolygon, startLine->getStartPoint(), startLine->getEndPoint());
								installTrack(buffer);
							}
							catch (ScException e) {
								e.printMsg();
								showLabel("Error installing track");
							}
						}
						break;
					case SDLK_u:
						buffer = getName(view);
						if (buffer == NULL)	break;
						try { uninstallTrack(buffer); }
						catch (ScException e) {
							e.printMsg();
							showLabel("Error uninstalling track");
						}
						break;
					case SDLK_h:
						showHelp();
						break;
					case SDLK_ESCAPE:
						running = false;
						break;
					default:
						break;
				}
			}
		}
		
		// set start line position
		if (drawingStartLine == true)
		{
			startLine->setLine(startLineA, Tuple(xloc,yloc));
		}
		
		// draw tile
		if (buttonState == true)
		{
			int col = int(xloc / gTiles->getTileWidth());
			int row = int(yloc / gTiles->getTileHeight());
			if (col >= 0 && col < gTiles->getMapWidth() &&
				row >= 0 && row < gTiles->getMapHeight())
			{
				unsigned char temp = (*gTiles)(col,row);
				
				if (rightB)
					(*gTiles)(col,row) = 25;
				else
					(*gTiles)(col,row) = (col % 5) + 5*(row % 5);
				
				if (temp != (*gTiles)(col,row)) {
					tileToggleCount++;
					if (tileToggleCount > kMinimapUpdateLimit) {
						gMinimap->updateMap(*gTiles);
						tileToggleCount = 0;
					}
				}
			}
		}
		
		// scroll
		if (scrollLeft)
			view.scrollBy(-1,0);
		if (scrollRight)
			view.scrollBy(1,0);
		if (scrollDown)
			view.scrollBy(0,-1);
		if (scrollUp)
			view.scrollBy(0,1);
		
		// bound scroll rect to tilemap bounds
		if (view.getViewRect().left < gTiles->getBounds().left)
			view.scrollBy(gTiles->getBounds().left - view.getViewRect().left, 0);
		if (view.getViewRect().bottom < gTiles->getBounds().bottom)
			view.scrollBy(0, gTiles->getBounds().bottom - view.getViewRect().bottom);
		if (view.getViewRect().right > gTiles->getBounds().right)
			view.scrollBy(gTiles->getBounds().right - view.getViewRect().right, 0);
		if (view.getViewRect().top > gTiles->getBounds().top)
			view.scrollBy(0, gTiles->getBounds().top - view.getViewRect().top);
	}
}

void installTrack(const char *name)
{
	// Load the xml file
	TiXmlDocument *xmlDoc = new TiXmlDocument("tracks/tracklist.xml");
	if (!xmlDoc->LoadFile())
	{
		delete xmlDoc;
		ScThrowErr("xml file not found");
	}
	
	// Add name to track list
	TiXmlHandle *docHandle = new TiXmlHandle(xmlDoc);
	TiXmlElement *xTracklist = docHandle->FirstChildElement("tracklist").Element();
	if (xTracklist == NULL)
		ScThrowErr("xml file could not find tracklist element");
    TiXmlElement track("track");
    track.SetAttribute("name", name);
	xTracklist->InsertEndChild(track);
	
	// Save the xml file
	if (!xmlDoc->SaveFile())
	{
		delete xmlDoc;
		ScThrowErr("error saving xml file");
	}
	
	//delete xTracklist;
	delete docHandle;
	delete xmlDoc;
	
	printf("install done\n");
	showLabel("Done");
}

void uninstallTrack(const char *name)
{
	// Load the xml file
	const char *attribute;
	TiXmlDocument *xmlDoc = new TiXmlDocument("tracks/tracklist.xml");
	if (!xmlDoc->LoadFile())
	{
		delete xmlDoc;
		ScThrowErr("xml file not found");
	}
	
	// Find the entry in the track list
	TiXmlHandle *docHandle = new TiXmlHandle(xmlDoc);
	TiXmlElement *xTracklist = docHandle->FirstChildElement("tracklist").Element();
	if (xTracklist == NULL)
		ScThrowErr("xml file could not find tracklist element");
    TiXmlElement *track = xTracklist->FirstChildElement("track");
	while (track != NULL)
	{
		attribute = track->Attribute("name");
		if (attribute == NULL)
			ScThrowErr("xml track element does not have attribute called \"name\"");
		if (strcmp(attribute, name) == 0)
			break;
		track = track->NextSiblingElement("track");
	}
	
	// Remove the track
	if (track == NULL) {
		printf("no track called %s is installed\n", name);
	}
	else {
		xTracklist->RemoveChild(track);
		
		// Save the xml file
		if (!xmlDoc->SaveFile())
		{
			delete xmlDoc;
			ScThrowErr("error saving xml file");
		}
	}
	
	// Remove actual files
	char xmlFileName[256];
	char mapFileName[256];
	char directoryName[256];
	sprintf(xmlFileName, "tracks/%s/%s.xml", name, name);
	
	// get the tilemap file name from the xml file
	TiXmlDocument *xmlTrackDoc = new TiXmlDocument(xmlFileName);
	if (!xmlTrackDoc->LoadFile())
	{
		delete xmlTrackDoc;
		ScThrowErr("xml file not found");
	}
	TiXmlHandle *trackDocHandle = new TiXmlHandle(xmlTrackDoc);
	TiXmlText *tilemapName = trackDocHandle->FirstChildElement("track").FirstChildElement("tilemap").FirstChild().ToText();
	if (tilemapName == NULL)
		ScThrowErr("xml tilemap text node not found");
	sprintf(mapFileName, "tracks/%s/%s", name, tilemapName->Value());
	
	delete trackDocHandle;
	delete xmlTrackDoc;
	
	sprintf(directoryName, "tracks/%s/", name);
	
	// remove the files
	if (remove(xmlFileName) == -1) {
		printf("removing the xml file failed\n");
	}
	if (remove(mapFileName) == -1) {
		printf("removing the map file failed\n");
	}
	if (remove(directoryName) == -1) {
		printf("removing the directory failed\n");
	}
	
	delete docHandle;
	delete xmlDoc;
	
	printf("uninstall done\n");
	showLabel("Done");
}

void saveTrack(const char *path, list<Tuple>& pathPolygon, Tuple startLineA, Tuple startLineB)
{
	char filename[256] = "";
	char *trackname;
	char pathPolygonStr[4096] = "";
	char tempStr[32];
	
	trackname = strrchr(path, '/');
	if (trackname)
		trackname++;
	else
		trackname = (char *)path;
	
	strcat(filename, path);
	strcat(filename, "/");
	strcat(filename, trackname);
	strcat(filename, ".xml");
	
	//mkdir(path);
	char callStr[256] = "mkdir -p ";
	strcat(callStr, path);
	system(callStr);
	
	// Setup the xml file
	printf("xml filename is %s\n", filename);
	TiXmlDocument *xmlDoc = new TiXmlDocument(filename);
	
    TiXmlDeclaration decl("1.0", "UTF-8", "yes");
	TiXmlElement xTrack("track");
	TiXmlElement xTileset("tileset");
	TiXmlElement xTilemap("tilemap");
	TiXmlElement xStartline("startline");
	TiXmlElement xPath("path");
	
	xTileset.InsertEndChild(TiXmlText("default"));
	
	startLineA.x /= gTiles->getTileWidth();
	startLineA.y /= gTiles->getTileHeight();
	startLineB.x /= gTiles->getTileWidth();
	startLineB.y /= gTiles->getTileHeight();
	xStartline.SetDoubleAttribute("x1", startLineA.x);
	xStartline.SetDoubleAttribute("y1", startLineA.y);
	xStartline.SetDoubleAttribute("x2", startLineB.x);
	xStartline.SetDoubleAttribute("y2", startLineB.y);
	
	strcpy(filename, trackname);
	strcat(filename, ".map");
	xTilemap.InsertEndChild(TiXmlText(filename));
	
	for (list<Tuple>::const_iterator i = pathPolygon.begin(); i != pathPolygon.end(); i++) {
		sprintf(tempStr, "%.2lf,%.2lf; ", i->x / gTiles->getTileWidth() * kScale, i->y / gTiles->getTileWidth() * kScale);
		strcat(pathPolygonStr, tempStr);
	}
	if (!pathPolygon.empty()) {
		pathPolygonStr[strlen(pathPolygonStr) - 2] = '\0';
		xPath.InsertEndChild(TiXmlText(pathPolygonStr));
	}
	else {
		xPath.InsertEndChild(TiXmlText(""));
	}
	
	xTrack.InsertEndChild(xTileset);
	xTrack.InsertEndChild(xTilemap);
	xTrack.InsertEndChild(xStartline);
	xTrack.InsertEndChild(xPath);
	
	strcpy(filename, path);
	strcat(filename, "/");
	strcat(filename, trackname);
	strcat(filename, ".map");
    xmlDoc->InsertEndChild(decl);
    xmlDoc->InsertEndChild(xTrack);
	xmlDoc->SaveFile();
	printf("Saving to file %s...\n", filename);
	try { gTiles->saveMap(filename); }
	catch (ScException e) {
		e.printMsg();
		SDL_Quit();
		exit(EXIT_FAILURE);
	}
	
	// Clear and delete our allocated document and return success ;)
	xmlDoc->Clear();
	delete xmlDoc;
	
	printf("saving done\n");
	showLabel("Done");
}

void loadTrack(const char *path, list<Tuple>& pathPolygon, Startline *startLine)
{
	char *trackname;
	char filename[256];
	
	trackname = strrchr(path, '/');
	if (trackname)
		trackname++;
	else
		trackname = (char *)path;
	
	strcpy(filename, path);
	strcat(filename, "/");
	strcat(filename, trackname);
	strcat(filename, ".xml");
	
	// Load the xml file
	TiXmlDocument *xmlDoc = new TiXmlDocument(filename);
	if (!xmlDoc->LoadFile())
	{
		delete xmlDoc;
		ScThrowErr("xml file not found");
	}
	
	TiXmlHandle docHandle(xmlDoc);
	
	TiXmlElement *xTrack = docHandle.FirstChildElement("track").Element();
	TiXmlText *xTileset = docHandle.FirstChildElement("track").FirstChild("tileset").FirstChild().ToText();
	TiXmlText *xTilemap = docHandle.FirstChildElement("track").FirstChild("tilemap").FirstChild().ToText();
	TiXmlElement *xStartline = docHandle.FirstChildElement("track").FirstChildElement("startline").Element();
	TiXmlText *xPath = docHandle.FirstChildElement("track").FirstChild("path").FirstChild().ToText();
	
	// do error checking
	if (!(xTrack && xTileset && xTilemap && xStartline))
	{
		xmlDoc->Clear();
		delete xmlDoc;
		if (!xTrack)
			ScThrowErr("xml track element not found");
		if (!xTileset)
			ScThrowErr("xml tileset element not found");
		if (!xTilemap)
			ScThrowErr("xml tilemap element not found");
		if (!xStartline)
			ScThrowErr("xml startline element not found");
		if (!xPath)
			ScThrowErr("xml path element not found");
	}
	
	// load the tilemap from the specified tilemap file
	strcpy(filename, path);
	strcat(filename, "/");
	strcat(filename, xTilemap->Value());
	try { gTiles->loadMap(filename); }
	catch (ScException e) {
		e.printMsg();
	}
	
	// the path
	pathPolygon.clear();
	if (xPath != NULL) {
		Tuple points[256];
		short pointsCount;
		readPointsFromString(xPath->Value(), points, pointsCount);
		for (short k = 0; k < pointsCount; k++) {
			pathPolygon.push_back(points[k] * gTiles->getTileWidth() / kScale);
		}
	}
	
	// get the start line attributes
	Tuple p1, p2;
	if (!(xStartline->Attribute("x1", &p1.x) && xStartline->Attribute("y1", &p1.y) &&
			xStartline->Attribute("x2", &p2.x) && xStartline->Attribute("y2", &p2.y)))
	{
		xmlDoc->Clear();
		delete xmlDoc;
		ScThrowErr("xml startline attributes not set");
	}
	p1.x *= gTiles->getTileWidth();
	p2.x *= gTiles->getTileWidth();
	p1.y *= gTiles->getTileHeight();
	p2.y *= gTiles->getTileHeight();
	
	startLine->setLine(p1, p2);
	
	// Clear and delete our allocated document and return success ;)
	xmlDoc->Clear();
	delete xmlDoc;
	
	printf("loading done\n");
	showLabel("Done");
}

void loadFromImage(const char *imagefilename)
{
	const int ROAD_COLOR_INDEX = 0;
	SDL_Surface	*surface;
	unsigned short width, height, bpp, pad_bytes;
	unsigned char value8;
	int i, j;
	unsigned char *cur;
	
	/* Try to load the image file */
	surface = IMG_Load(imagefilename);
	if (surface == NULL)
	{
		ScThrowErr("Error opening image file");
	}
	
	/* calculate some numbers */
	width = surface->w;
	height = surface->h;
	bpp = surface->format->BytesPerPixel;
	/*pad_bytes = (4 - (width%4)) % 4;*/
	pad_bytes = surface->pitch/bpp - width;
	
	//printf("testing: %d should be equal to %d\n", pad_bytes, (4 - (width%4)) % 4);
	
	if (bpp != 1)
	{
		printf("Image must be 256 colours only\n");
		exit(1);
	}
	
	try { gTiles->newMap(width, height); }
	catch (ScException e) {
		e.printMsg();
		SDL_Quit();
		exit(EXIT_FAILURE);
	}
	
	/* Write map tile data to tilemap */
	cur = (unsigned char *)surface->pixels;
	for (i = 0; i < height; i++)
	{
		for (j = 0; j < width; j++)
		{
			value8 = (*cur == ROAD_COLOR_INDEX) ? 25 : 5*(i%5) + (j%5);
			(*gTiles)(j,i) = value8;
			cur += bpp;
		}
		cur += pad_bytes;
	}
	
	SDL_FreeSurface(surface);
	
	printf("loading from image done\n");
	showLabel("Done");
	
    /* OK, everything went fine - return... */
	return;
}
